#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-10-28
#
# Monitor status of Cisco flash file systems
#
# 2019-10-28: initial release
# 2019-11-04: added wato for cisco_flash
# 2021-07-31: rewritten for CMK 2.0
#
# snmpwalk sample
#
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.2.1.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.2.2.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.3.1.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.3.2.1 = INTEGER: 1
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.4.1.1 = Gauge32: 2908606464
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.4.2.1 = Gauge32: 4294967295
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.5.1.1 = Gauge32: 1756741632
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.5.2.1 = Gauge32: 4294967295
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.6.1.1 = Gauge32: 624
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.6.2.1 = Gauge32: 0
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.7.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.7.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.8.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.8.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.9.1.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.9.2.1 = INTEGER: 3
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.1.1 = STRING: "bootflash"
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.2.1 = STRING: "usb0"
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.11.1.1 = INTEGER: 2
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.11.2.1 = INTEGER: 2
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.12.1.1 = INTEGER: 63
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.12.2.1 = INTEGER: 63
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.13.1.1 = Counter64: 2908606464
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.13.2.1 = Counter64: 16424566784
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.14.1.1 = Counter64: 1756741632
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.14.2.1 = Counter64: 16424558592
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.15.1.1 = INTEGER: 10
# .1.3.6.1.4.1.9.9.10.1.1.4.1.1.15.2.1 = INTEGER: 10
#
# sample info
#
# [
#  [u'1.1', u'2908606464', u'1758527488', u'446', u'3', u'bootflash', u'2', u'2908606464', u'1758527488', u'10'],
#  [u'2.1', u'4294967295', u'4294967295', u'0', u'3', u'usb0', u'2', u'16423845888', u'7506280448', u'10']
# ]
#
#

from dataclasses import dataclass
from typing import Optional, Dict
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    SNMPTree,
    contains,
    check_levels,
    OIDEnd,
    Result,
    render,
    State,
    Metric,
    all_of,
    exists,
)


@dataclass
class Partition:
    size: Optional[int] = None
    freespace: Optional[int] = None
    usedspace: Optional[int] = None
    index: Optional[int] = None
    flashindex: Optional[int] = None
    neederasure: Optional[bool] = None
    lowspacenotifythreshold: Optional[int] = None
    filecount: Optional[int] = None


_partneederasure = {
    '1': True,
    '2': False,
}


###########################################################################
#
#  DATA Parser function
#
###########################################################################


def parse_cisco_flash(string_table: StringTable) -> Dict[str, Partition]:
    partitions: Dict[str, Partition] = {}

    for cflPartition in string_table:
        cflpartOID_END, cflpartSize, cflpartFreeSpace, cflpartFileCount, cflpartName, cflpartNeedErasure, \
        cflpartSizeExtended, cflpartFreeSpaceExtended, cflpartLowSpaceNotifThreshold = cflPartition

        cflDevIndex, cflPartIndex = cflpartOID_END.split('.')

        cflpartFreeSpace = int(cflpartFreeSpace) if cflpartFreeSpace.isdigit() else None
        cflpartFreeSpaceExtended = int(cflpartFreeSpaceExtended) if cflpartFreeSpaceExtended.isdigit() else None
        cflpartSize = int(cflpartSize) if cflpartSize.isdigit() else None
        cflpartSizeExtended = int(cflpartSizeExtended) if cflpartSizeExtended.isdigit() else None

        freespace = cflpartFreeSpaceExtended if cflpartFreeSpaceExtended else cflpartFreeSpace
        size = cflpartSizeExtended if cflpartSizeExtended else cflpartSize

        if size > 0:
            partitions.update({
                cflpartName: Partition(
                    index=int(cflPartIndex) if cflPartIndex.isdigit() else None,
                    flashindex=int(cflDevIndex) if cflDevIndex.isdigit() else None,
                    size=size,
                    freespace=freespace,
                    usedspace=size - freespace,
                    filecount=int(cflpartFileCount) if cflpartFileCount.isdigit() else None,
                    neederasure=_partneederasure.get(cflpartNeedErasure, None),
                    lowspacenotifythreshold=int(
                        cflpartLowSpaceNotifThreshold) if cflpartLowSpaceNotifThreshold.isdigit() else None
                )})

    return partitions


###########################################################################
#
#  INVENTORY function
#
###########################################################################


def discovery_cisco_flash(section: Dict[str, Partition]) -> DiscoveryResult:
    for key in section:
        yield Service(item=key)


###########################################################################
#
#  CHECK function
#
###########################################################################


def check_cisco_flash(item, params, section: Dict[str, Partition]) -> CheckResult:
    try:
        partition = section[item]
    except KeyError:
        return

    if partition.lowspacenotifythreshold:
        spacewarn = partition.size - ((partition.size / 100) * partition.lowspacenotifythreshold)
    else:
        spacewarn = partition.size

    yield from check_levels(
        value=partition.usedspace,
        label='Space used',
        metric_name='cisco_flash_partusedspace',
        render_func=render.disksize,
        levels_upper=(spacewarn, partition.size - 10),
        boundaries=(0, partition.size),
    )

    percentused = 100 * partition.usedspace / partition.size

    yield from check_levels(
        value=percentused,
        label='Percent',
        levels_upper=params['levels_upper_percent'],
        render_func=render.percent,
        boundaries=(0, 100),
        metric_name='cisco_flash_percent_used'
    )

    if partition.neederasure:
        yield Result(state=State.WARN, notice='Partition needs erasure')

    yield Metric(
        value=partition.filecount,
        name='cisco_flash_partfilecount',
        boundaries=(0, None)
    )

    for key, value in [
        ('Device index', partition.flashindex),
        ('Partition index', partition.index),
        ('Partition need erasure', partition.neederasure),
        ('Partition low space notify threshold', partition.lowspacenotifythreshold),
    ]:
        if value:
            yield Result(state=State.OK, notice=f'{key}: {value}')

    size = max(partition.size, 1)  # for size = 0 --> div/0
    size = size / 1000 / 1000
    usedspace = partition.usedspace / 1000 / 1000
    freespace = partition.freespace / 1000 / 1000

    yield Result(state=State.OK, notice=f'Used: {usedspace:.2f} of {size:.2f} MB, {freespace:.2f} MB free')

    if partition.filecount:
        yield Result(state=State.OK, summary=f'File count: {partition.filecount}')


###########################################################################
#
#  SNMP section
#
###########################################################################


register.snmp_section(
    name='cisco_flash',
    parse_function=parse_cisco_flash,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.9.9.10.1.1.4.1.1',  #
        oids=[
            OIDEnd(),
            '4',  # ciscoFlashPartitionSize
            '5',  # ciscoFlashPartitionFreeSpace
            '6',  # ciscoFlashPartitionFileCount
            '10',  # ciscoFlashPartitionName
            '11',  # ciscoFlashPartitionNeedErasure
            '13',  # ciscoFlashPartitionSizeExtended
            '14',  # ciscoFlashPartitionFreeSpaceExtended
            '15',  # ciscoFlashPartitionLowSpaceNotifThreshold
        ]
    ),
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),  # sysDescr
        exists('.1.3.6.1.4.1.9.9.10.1.1.4.1.1.10.*'),  # CISCO-FLASH-MIB::ciscoFlashDevicesSupported
    ))

register.check_plugin(
    name='cisco_flash',
    service_name='Flash partition %s',
    discovery_function=discovery_cisco_flash,
    check_function=check_cisco_flash,
    check_default_parameters={
        'levels_upper_percent': (85, 90),
    },
    check_ruleset_name='cisco_flash',
)
