#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-10-22 
#
# invetory of cisco flash devices
#
# 2019-10-22 : initial release
# 2021-07-31: rewritten for CMK 2.0
#
#

from typing import List, Dict, Any
from dataclasses import dataclass
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringTable,
    InventoryResult,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    OIDEnd,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    TableRow,
    exists,
    all_of,
    contains,
)


@dataclass
class Section:
    flash_devices: List[Dict[str, Any]]
    flash_chips: List[Dict[str, Any]]
    flash_partitions: List[Dict[str, Any]]


_DevProgrammingJumper = {
    '1': 'installed',
    '2': 'not installed',
    '3': 'unknown',
}

_DevRemovable = {
    '1': True,
    '2': False,
}

_PartStatus = {
    '1': 'Read Only',
    '2': 'run from flash',
    '3': 'Read Write',
}

_PartChecksumAlgorithm = {
    '1': 'simple Checksum',
    '2': 'undefined',
    '3': 'simple CRC',
}

_PartUpgradeMethod = {
    '1': 'unkown',
    '2': 'rxboot Flash Load Helper',
    '3': 'direct',
}

_PartNeedErasure = {
    '1': True,
    '2': False,
}


def parse_inv_cisco_flash(string_table: List[StringTable]) -> Section:
    cflDevices, cflChips, cflPartitions = string_table

    flash_devices = []
    for cflDevice in cflDevices:
        clfOID_END, cflDevSize, cflDevMinPartitionSize, cflDevMaxPartitions, cflDevPartitions, cflDevChipCount, \
        cflDevName, cflDevDescr, cflDevController, cflDevProgrammingJumper, cflDevInitTime, cflDevRemovable, \
        cflDevPhyEntIndex, cflDevNameExtended, cflDevSizeExtended, cflDevMinPartitionSizeExtended = cflDevice

        cflDevSize = int(cflDevSize) if cflDevSize.isdigit() else None
        cflDevSizeExtended = int(cflDevSizeExtended) if cflDevSizeExtended.isdigit() else None
        cflDevMinPartitionSize = int(cflDevMinPartitionSize) if cflDevMinPartitionSize.isdigit() else None
        cflDevMinPartitionSizeExtended = int(
            cflDevMinPartitionSizeExtended) if cflDevMinPartitionSizeExtended.isdigit() else None

        # change sizes to MB
        size = cflDevSizeExtended if cflDevSizeExtended else cflDevSize
        size = size / 1024 / 1024 if size else None

        minpartitionsize = cflDevMinPartitionSizeExtended if cflDevMinPartitionSizeExtended else cflDevMinPartitionSize
        minpartitionsize = minpartitionsize / 1024 / 1024 if minpartitionsize else None

        flash_device = {}
        for key, value in [
            ('index', clfOID_END),
            ('name', cflDevName),
            ('description', cflDevDescr),
            ('controller', cflDevController),
            ('programmingjumper', _DevProgrammingJumper.get(cflDevProgrammingJumper)),
            ('inittime', cflDevInitTime),
            ('removable', _DevRemovable.get(cflDevRemovable)),
            ('nameextended', cflDevNameExtended),
            ('physentindex', cflDevPhyEntIndex),
            ('size', size),
            ('minpartitionsize', minpartitionsize),
            ('maxprtitions', cflDevMaxPartitions),
            ('partitions', cflDevPartitions),
            ('chipcount', cflDevChipCount),
        ]:
            if value:
                flash_device.update({key: value})
        flash_devices.append(flash_device)

    flash_chips = []

    for cflChip in cflChips:
        cflOID_END, cflChipCode, cflChipDescr, cflChipWriteRetries, cflChipEraseRetries, cflChipMaxWriteRetries, \
        cflChipMaxEraseRetries = cflChip

        cflDevIndex, cflChipIndex = cflOID_END.split('.')

        flash_chip = {}
        for key, value in [
            ('index', cflChipIndex),
            ('flashindex', cflDevIndex),
            ('code', cflChipCode),
            ('description', cflChipDescr),
            ('writeretries', cflChipWriteRetries),
            ('eraseretries', cflChipEraseRetries),
            ('maxwriteretries', cflChipMaxWriteRetries),
            ('maxeraseretries', cflChipMaxEraseRetries),
        ]:
            if value:
                flash_chip.update({key: value})

        flash_chips.append(flash_chip)

    flash_partitions = []
    for cflPartition in cflPartitions:
        cflpartOID_END, cflpartStartChip, cflpartEndChip, cflpartSize, cflpartFreeSpace, cflpartFileCount, \
        cflpartChecksumAlgorithm, cflpartStatus, cflpartUpgradeMethod, cflpartName, cflpartNeedErasure, \
        cflpartFileNameLength, cflpartSizeExtended, cflpartFreeSpaceExtended, \
        cflpartLowSpaceNotifThreshold = cflPartition

        cflDevIndex, cflPartIndex = cflpartOID_END.split('.')

        cflpartSize = int(cflpartSize) if cflpartSize.isdigit() else None
        cflpartFreeSpace = int(cflpartFreeSpace) if cflpartFreeSpace.isdigit() else None
        cflpartSizeExtended = int(cflpartSizeExtended) if cflpartSizeExtended.isdigit() else None
        cflpartFreeSpaceExtended = int(cflpartFreeSpaceExtended) if cflpartFreeSpaceExtended.isdigit() else None

        # change sizes to MB
        size = cflpartSizeExtended if cflpartSizeExtended else cflpartSize
        size = size / 1024 / 1024 if size else None

        freespace = cflpartFreeSpaceExtended if cflpartFreeSpaceExtended else cflpartFreeSpace
        freespace = freespace / 1024 / 1024 if freespace else None

        flash_partition = {}
        for key, value in [
            ('index', cflPartIndex),
            ('flashindex', cflDevIndex),
            ('startchip', cflpartStartChip),
            ('endchip', cflpartEndChip),
            ('filecount', cflpartFileCount),
            ('crcsumalgo', _PartChecksumAlgorithm.get(cflpartChecksumAlgorithm)),
            ('status', _PartStatus.get(cflpartStatus)),
            ('upgrademethod', _PartUpgradeMethod.get(cflpartUpgradeMethod)),
            ('name', cflpartName),
            ('neederasure', _PartNeedErasure.get(cflpartNeedErasure)),
            ('filenamelength', cflpartFileNameLength),
            ('lowspacenotifythreshold', cflpartLowSpaceNotifThreshold),
            ('size', size),
            ('freespace', freespace),
        ]:
            if value:
                flash_partition.update({key: value})

        flash_partitions.append(flash_partition)

    return Section(
        flash_devices=flash_devices,
        flash_chips=flash_chips,
        flash_partitions=flash_partitions
    )


def inventory_cisco_flash(params, section: Section) -> InventoryResult:
    flash_devices = section.flash_devices
    flash_chips = section.flash_chips
    flash_partitions = section.flash_partitions

    # inventory of cisco flash devices
    removecolumns = []
    disabled = False
    # 
    if params:
        disabled = params.get('cflDeviceDisable', disabled)
        # get list of columns to remove from inventory
        removecolumns = params.get('cflDeviceRemovecolumns', removecolumns)

    if not disabled:

        path = ['hardware', 'components', 'flash', 'devices']

        for flash_device in flash_devices:
            key_columns = {'index': flash_device['index'], }
            flash_device.pop('index')
            for key in flash_device.keys():
                if key in removecolumns:
                    flash_device.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_device
            )

    # inventory of cisco flash chips

    removecolumns = []
    disabled = False

    if params:
        disabled = params.get('cflChipDisable', disabled)
        # get list of columns to remove from inventory
        removecolumns = params.get('cflChipRemovecolumns', removecolumns)

    if not disabled:
        path = ['hardware', 'components', 'flash', 'chips']

        for flash_chip in flash_chips:
            key_columns = {'index': flash_chip['index'], 'flashindex': flash_chip['flashindex']}
            flash_chip.pop('index')
            flash_chip.pop('flashindex')
            for key in flash_chip.keys():
                if key in removecolumns:
                    flash_chip.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_chip
            )

    # inventory of cisco flash partitions

    removecolumns = []
    disabled = False

    if params:
        disabled = params.get('cflPartitionDisable', disabled)
        # get list of columns to remove from inventory
        removecolumns = params.get('cflPartitionRemovecolumns', removecolumns)

    if not disabled:
        path = ['hardware', 'components', 'flash', 'partitions']

        for flash_partition in flash_partitions:
            key_columns = {'index': flash_partition['index'], 'flashindex': flash_partition['flashindex']}
            flash_partition.pop('index')
            flash_partition.pop('flashindex')
            for key in flash_partition.keys():
                if key in removecolumns:
                    flash_partition.pop(key)

            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=flash_partition
            )


register.snmp_section(
    name='inv_cisco_flash',
    parse_function=parse_inv_cisco_flash,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.2.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashDeviceSize
                '3',  # ciscoFlashDeviceMinPartitionSize
                '4',  # ciscoFlashDeviceMaxPartitions
                '5',  # ciscoFlashDevicePartitions
                '6',  # ciscoFlashDeviceChipCount
                '7',  # ciscoFlashDeviceName
                '8',  # ciscoFlashDeviceDescr
                '9',  # ciscoFlashDeviceController
                '11',  # ciscoFlashDeviceProgrammingJumper
                '12',  # ciscoFlashDeviceInitTime
                '13',  # ciscoFlashDeviceRemovable
                '14',  # ciscoFlashPhyEntIndex
                '15',  # ciscoFlashDeviceNameExtended
                '16',  # ciscoFlashDeviceSizeExtended
                '17',  # ciscoFlashDeviceMinPartitionSizeExtended
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.3.1.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashChipCode
                '3',  # ciscoFlashChipDescr
                '4',  # ciscoFlashChipWriteRetries
                '5',  # ciscoFlashChipEraseRetries
                '6',  # ciscoFlashChipMaxWriteRetries
                '7',  # ciscoFlashChipMaxEraseRetries
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.10.1.1.4.1.1',  #
            oids=[
                OIDEnd(),  # ifIndex
                '2',  # ciscoFlashPartitionStartChip
                '3',  # ciscoFlashPartitionEndChip
                '4',  # ciscoFlashPartitionSize
                '5',  # ciscoFlashPartitionFreeSpace
                '6',  # ciscoFlashPartitionFileCount
                '7',  # ciscoFlashPartitionChecksumAlgorithm
                '8',  # ciscoFlashPartitionStatus
                '9',  # ciscoFlashPartitionUpgradeMethod
                '10',  # ciscoFlashPartitionName
                '11',  # ciscoFlashPartitionNeedErasure
                '12',  # ciscoFlashPartitionFileNameLength
                '13',  # ciscoFlashPartitionSizeExtended
                '14',  # ciscoFlashPartitionFreeSpaceExtended
                '15',  # ciscoFlashPartitionLowSpaceNotifThreshold
            ]
        ),
    ],
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),
        exists('.1.3.6.1.4.1.9.9.10.1.1.1.*'),  # CISCO-FLASH-MIB::ciscoFlashDevicesSupported
    )
)

register.inventory_plugin(
    name='inv_cisco_flash',
    inventory_function=inventory_cisco_flash,
    inventory_default_parameters={},
    inventory_ruleset_name='inv_cisco_flash',
)
